@@ -117,7 +117,7 @@ function btest {
   fi
 }
 
-ntests=29
+ntests=30
 declare -a tests[$ntests]
 declare -a result[$ntests]
 tests[1]="print 'hi'"
@@ -196,6 +196,9 @@ result[28]='ok'
 # use test
 tests[29]='use IO;print "ok"'
 result[29]='ok'
+# run-time context of ..
+tests[30]='@a=(4,6,1,0,0,1);sub range{(shift @a)..(shift @a)}print range();while(@a){print scalar(range())}'
+result[30]='456123E0'
 
 
 while getopts "qscCh" opt
